"""
This is a suite of automated tests, meant to be run on various quantifiable parts of this project to ensure they work as
intended.
"""
import os
import sys
import unittest

# Append parent directory to enhance portability.
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))
